#!/usr/bin/env python
# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: Copyright 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import json
from datetime import datetime
import logging
import fosslight_util.constant as constant
from fosslight_util.set_log import init_log
from ._help import print_help_msg_doubleopen, print_version
from fosslight_util.output_format import write_output_file
from ._package_item import PackageItem
import argparse

logger = logging.getLogger(constant.LOGGER_NAME)
_PKG_NAME = "fosslight_yocto"


def copy_info_from_recipe(pkg, recipe):
    if recipe != "" and recipe is not None:
        pkg.comment = recipe.comment
        pkg.download_location = recipe.download_location
        pkg.license = recipe.license
        pkg.copyright = recipe.copyright
        pkg.homepage = recipe.homepage
        pkg.oss_name = recipe.oss_name


def matching_data_from_recipe(pkg_list, recipe_list, generated_list, is_distributed=False):
    rc = True
    msg = ""

    try:
        for pkg in pkg_list:
            pkg_spdx = pkg.spdx_id
            logger.debug("FIND : " + pkg_spdx)
            if pkg_spdx in generated_list:
                recipe = generated_list[pkg_spdx]
                logger.debug("|-- RECIPE : " + recipe)
                try:
                    found_idx = recipe_list.index(recipe)
                    recipe_item = recipe_list[found_idx]
                    if is_distributed:
                        recipe_item.exclude = False
                    pkg = copy_info_from_recipe(pkg, recipe_item)
                except ValueError as ex:
                    logger.debug(f"|--- Can't find recipe_ {recipe}:{ex}")
                    continue

    except Exception as ex:
        msg = f"Error Parsing item:{ex}"
        rc = False
        logger.error(msg)

    return rc, msg, pkg_list


def parsing_pkg_item(pkg_list, pkg_distributed):

    rc = True
    pkg_items = []
    recipe_items = []
    pkg_notDistributed = []
    logger.debug("TOTAL FILE COUNT: " + str(len(pkg_list)) + "\n")

    for pkg in pkg_list:
        try:
            pkg_item = PackageItem()
            for key, value in pkg.items():
                if key == 'versionInfo':
                    pkg_item.version = value
                elif key == 'downloadLocation':
                    pkg_item.download_location = value
                elif key == 'licenseDeclared':
                    pkg_item.license = value
                elif key == 'file':
                    pkg_item.source_name_or_path = value
                elif key == 'copyrightText':
                    pkg_item.copyright = value
                elif key == 'comment':
                    pkg_item.comment = value
                elif key == 'name':
                    pkg_item.package_name = value
                    pkg_item.oss_name = value
                    pkg_item.parent_package_name = value
                elif key == 'homepage':
                    pkg_item.homepage = value
                elif key == 'SPDXID':
                    pkg_item.spdx_id = value

            if pkg_item.spdx_id.startswith("SPDXRef-Recipe-"):
                pkg_item.exclude = True
                recipe_items.append(pkg_item)
            else:
                if pkg_item.spdx_id in pkg_distributed:
                    pkg_items.append(pkg_item)
                else:
                    pkg_item.exclude = True
                    pkg_notDistributed.append(pkg_item)

        except Exception as ex:
            rc = False
            logger.error(f"Error Parsing item:{ex}")

    logger.info(f"- TOTAL Distributed: {len(pkg_items)}")
    logger.info(f"- TOTAL Recipe: {len(recipe_items)}")
    logger.info(f"- TOTAL Not Distributed: {len(pkg_notDistributed)}")

    return rc, pkg_items, recipe_items, pkg_notDistributed


def parsing_relationships_item(relationships_items):
    rc = True
    spdx_distributed = []
    generated_list = {}

    for rel_item in relationships_items:
        try:
            rel_type = ""
            spdx_id = ""
            relatedSpdxElement = ""
            if "spdxElementId" in rel_item:
                spdx_id = rel_item["spdxElementId"]
            if "relationshipType" in rel_item:
                rel_type = rel_item["relationshipType"]
            if "relatedSpdxElement" in rel_item:
                relatedSpdxElement = rel_item["relatedSpdxElement"]
            if spdx_id:
                if rel_type == "PACKAGE_OF":
                    spdx_distributed.append(spdx_id)
                if rel_type == "GENERATES":  # CONTAINS GENERATED_FROM
                    generated_list[relatedSpdxElement] = spdx_id

        except Exception as ex:
            rc = False
            logger.error(f"Parsing relation:{ex}")

    return rc, spdx_distributed, generated_list


def parsing_spdx_json(json_file):
    recipes = []
    pkg_distributed = []
    pkg_notDistributed = []
    try:
        logger.info("Start parsing " + json_file)
        with open(json_file, "r") as st_json:
            st_python = json.load(st_json)
            rc, spdx_distributed, generated_list = parsing_relationships_item(st_python["relationships"])
            rc, pkg_distributed, recipes, pkg_notDistributed = parsing_pkg_item(st_python["packages"], spdx_distributed)
            rc, msg, pkg_distributed = matching_data_from_recipe(pkg_distributed, recipes, generated_list, True)
            rc, msg, pkg_notDistributed = matching_data_from_recipe(pkg_notDistributed, recipes, generated_list)

    except Exception as error:
        logger.error(f"Parsing {json_file}:{error}")
    return recipes, pkg_distributed, pkg_notDistributed


def get_sheet_content_to_print(pkg_list):
    list_to_print = []
    for item in pkg_list:
        list_to_print.extend(item.get_print_item())
    return list_to_print


def read_spdx_json(json_file_to_parse, output_file_name):
    recipes = []
    pkg_distributed = []
    pkg_notDistributed = []

    msg = ""
    success = True

    try:
        sheet_list = {}
        if os.path.isfile(json_file_to_parse):
            recipes, pkg_distributed, pkg_notDistributed = parsing_spdx_json(
                json_file_to_parse)

            sheet_list["SRC_distributed"] = get_sheet_content_to_print(pkg_distributed)
            sheet_list["SRC_recipe"] = get_sheet_content_to_print(recipes)
            sheet_list["SRC_not_distributed"] = get_sheet_content_to_print(pkg_notDistributed)
        else:
            msg = "Can't find a spdx json file:" + json_file_to_parse
            success = False

        success_to_write, writing_msg, result_file = write_output_file(output_file_name, ".xlsx", sheet_list)
        if success_to_write:
            logger.info(f"FOSSLight Report:{result_file}")
        else:
            logger.warning(f"Writing excel :{success_to_write}, {writing_msg}")

    except Exception as ex:
        success = False
        msg = str(ex)
        logger.error(msg)

    return msg, success


def main():
    global logger

    start_time = datetime.now().strftime('%y%m%d_%H%M')
    output_file_name = ""

    parser = argparse.ArgumentParser(description='FOSSLight Yocto', prog='fosslight_yocto', add_help=False)
    parser.add_argument('-h', '--help', action='store_true', required=False)
    parser.add_argument('-v', '--version', action='store_true', required=False)
    parser.add_argument('-o', '--output', type=str, required=False)
    parser.add_argument('-f', '--file', type=str, required=False)

    args = parser.parse_args()
    if args.help:
        print_help_msg_doubleopen()
    if args.version:
        print_version(_PKG_NAME)
    if args.file:
        spdx_file = args.file
    if args.output:
        output_file_name = os.path.abspath(args.output)

    if output_file_name == "":
        output_dir = os.getcwd()
        oss_report_name = f"fosslight_report_yocto_{start_time}"
    else:
        oss_report_name = output_file_name
        output_dir = os.path.dirname(output_file_name)

    logger, log_item = init_log(os.path.join(output_dir, f"fosslight_log_yocto_{start_time}.txt"))
    read_spdx_json(spdx_file, oss_report_name)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: Copyright 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import fosslight_util.constant as constant
import codecs
from ruamel.yaml import YAML
from ruamel.yaml.constructor import SafeConstructor
import os
from ._package_item import PackageItem, set_value_switch, update_package_name
from .parsing_yaml import parsing_yml
from copy import deepcopy

logger = logging.getLogger(constant.LOGGER_NAME)
KEY_PKG = "yocto_package"
KEY_RECIPE = "yocto_recipe"
MSG_TO_EXCLUDE = "Excluded by oss-pkg-info.yaml"
MSG_FROM_YAML_ROW = "Info loaded from oss-pkg-info.yaml"


def construct_yaml_map(self, node):
    # test if there are duplicate node keys
    keys = set()
    for key_node, value_node in node.value:
        key = self.construct_object(key_node, deep=True)
        if key in keys:
            break
        keys.add(key)
    else:
        data = {}
        yield data
        value = self.construct_mapping(node)
        data.update(value)
        return
    data = []
    yield data
    for key_node, value_node in node.value:
        key = self.construct_object(key_node, deep=True)
        val = self.construct_object(value_node, deep=True)
        data.append({key: val})


def set_list_by_value(value):
    items = []
    if value:
        if isinstance(value, list):
            items += value
        else:
            items = [value]
    return items


def load_oss_pkg_info_yaml(oss_pkg_files, print_bin_android_mode, installed_packages_src, installed_packages_bin, nested_pkg_name):
    all_pkgs_to_exclude = []
    oss_items_to_append = []
    for oss_pkg_info_yaml_file in oss_pkg_files.split(','):
        if os.path.isfile(oss_pkg_info_yaml_file):
            try:  # LATEST YAML FORMAT
                temp_items, _, _ = parsing_yml(oss_pkg_info_yaml_file, os.path.dirname(oss_pkg_info_yaml_file), False)
                for parent in temp_items:
                    child = PackageItem()
                    child.__dict__.update(parent.__dict__)
                    oss_items_to_append.append(child)
            except Exception as ex:
                logger.debug(f"Failed to parsing yaml({oss_pkg_info_yaml_file}): {ex}")
            if not oss_items_to_append:
                try:  # LEGACY YAML FORMAT
                    with codecs.open(oss_pkg_info_yaml_file, "r", "utf-8") as f:
                        data = f.read()
                        SafeConstructor.add_constructor(u'tag:yaml.org,2002:map', construct_yaml_map)
                        yaml = YAML(typ='safe')
                        data = yaml.load(data)

                        if not isinstance(data, list):
                            data_list = [data]
                        else:
                            data_list = data

                        for recipe_items in data_list:
                            for root_name, oss_items in recipe_items.items():
                                recipe_name = root_name.strip()
                                if oss_items:
                                    for item in oss_items:
                                        pkg_item = PackageItem()
                                        for key, value in item.items():
                                            set_value_switch(pkg_item, key, value, nested_pkg_name)
                                        if recipe_name != "Open Source Package":
                                            pkg_item.yocto_recipe = recipe_name
                                        oss_items_to_append.append(pkg_item)
                except Exception as ex:
                    logger.warning(f"Failed to read {oss_pkg_info_yaml_file}: {ex}")
        else:
            logger.warning(f"Can't find a file:{oss_pkg_info_yaml_file}")

    # Update oss item based on value(yocto_package, yocto_recipe) in item.
    pkg_item_list = []
    for oss in oss_items_to_append:
        recipe_names_to_search = oss.yocto_recipe
        pkg_names_to_search = oss.yocto_package
        # Find the item to be excluded by the recipe or package name.
        pkgs_to_exclude = [(x.package_name, x.parent_package_name, x.oss_name) for x in installed_packages_src if
                           x.oss_name in recipe_names_to_search or x.name in recipe_names_to_search
                           or x.parent_package_name in pkg_names_to_search or x.package_name in pkg_names_to_search]

        if len(pkgs_to_exclude) > 0:
            search_list = recipe_names_to_search + pkg_names_to_search
            len_search = len(search_list)
            if len_search > 0:
                update_package_name(oss, search_list[0], nested_pkg_name)
            elif len_search > 1:
                prev_comment = ""
                if "comment" in oss:
                    prev_comment = f"{oss.comment}/"
                oss.comment = f"{prev_comment}pkg, recipe:{search_list}"

            all_pkgs_to_exclude += pkgs_to_exclude

            oss.source_name_or_path = ""  # Don't print file value loaded from yaml file.
            oss.comment = MSG_FROM_YAML_ROW
            oss_name_from_yaml = oss.name

            for _, pkg_name, recipe in pkgs_to_exclude:
                oss.parent_package_name = pkg_name
                oss.oss_name = recipe
                oss.name = oss_name_from_yaml
                pkg_updated = deepcopy(oss)
                pkg_item_list.append(pkg_updated)

    # Exclude packages
    for pkg_name, _, _ in all_pkgs_to_exclude:
        pkg_items = list(filter(lambda x: x.package_name == pkg_name
                                and x.exclude is False, installed_packages_bin))
        pkg_items += [x for x in installed_packages_src if x.package_name == pkg_name and x.exclude is False]

        for item in pkg_items:
            item.exclude = True
            item.comment = MSG_TO_EXCLUDE

    # Append yaml's OSS items to SRC tab
    installed_packages_src.extend(pkg_item_list)

    return installed_packages_src, installed_packages_bin



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import yaml
import logging
import codecs
import os
import fosslight_util.constant as constant
from ._package_item import PackageItem

_logger = logging.getLogger(constant.LOGGER_NAME)
EXAMPLE_OSS_PKG_INFO_LINK = "https://github.com/fosslight/fosslight_prechecker/blob/main/tests/convert/sbom-info.yaml"


def parsing_yml(yaml_file, base_path, print_log=True):
    oss_list = []
    license_list = []
    idx = 1
    err_reason = ""
    OLD_YAML_ROOT_ELEMENT = ['Open Source Software Package',
                             'Open Source Package']

    try:
        path_of_yml = os.path.normpath(os.path.dirname(yaml_file))
        base_normpath = os.path.normpath(base_path)
        relative_path = ""
        if path_of_yml != base_normpath:
            relative_path = os.path.relpath(path_of_yml, base_normpath)
        else:
            relative_path = ""
        doc = yaml.safe_load(codecs.open(yaml_file, "r", "utf-8"))
        # If yaml file is empty, return immediately
        if doc is None:
            err_reason = "empty"
            if print_log:
                _logger.warning(f"The yaml file is empty file: {yaml_file}")
            return oss_list, license_list, err_reason

        is_old_format = any(x in doc for x in OLD_YAML_ROOT_ELEMENT)

        for root_element in doc:
            oss_items = doc[root_element]
            if oss_items:
                if not isinstance(oss_items, list) or 'version' not in oss_items[0]:
                    raise AttributeError(f"- Ref. {EXAMPLE_OSS_PKG_INFO_LINK}")
                for oss in oss_items:
                    item = PackageItem()
                    item.relative_path = relative_path
                    if not is_old_format:
                        item.name = root_element
                    for key, value in oss.items():
                        if key:
                            key = key.lower().strip()
                        set_value_switch(item, key, value, yaml_file)
                    oss_list.append(item)
                    license_list.extend(item.license)
                    idx += 1
    except AttributeError as ex:
        if print_log:
            _logger.warning(f"Not supported yaml file format: {yaml_file} {ex}")
        oss_list = []
        err_reason = "not_supported"
    except yaml.YAMLError:
        if print_log:
            _logger.warning(f"Error to parse yaml - skip to parse yaml file: {yaml_file}")
        oss_list = []
        err_reason = "yaml_error"
    return oss_list, set(license_list), err_reason


def set_value_switch(oss, key, value, yaml_file=""):
    if key in ['oss name', 'name']:
        oss.name = value
    elif key in ['oss version', 'version']:
        oss.version = value
    elif key in ['download location', 'source']:
        oss.download_location = value
    elif key in ['license', 'license text']:
        oss.license = value
    elif key in ['file name or path', 'source name or path', 'source path',
                 'file', 'binary name', 'binary path']:
        oss.source_name_or_path = value
    elif key in ['copyright text', 'copyright']:
        oss.copyright = value
    elif key == 'exclude':
        oss.exclude = value
    elif key == 'comment':
        oss.comment = value
    elif key == 'homepage':
        oss.homepage = value
    elif key == 'yocto_package':
        oss.yocto_package = value
    elif key == 'yocto_recipe':
        oss.yocto_recipe = value
    elif key == 'vulnerability link':
        oss.bin_vulnerability = value
    elif key == 'tlsh':
        oss.bin_tlsh = value
    elif key == 'sha1':
        oss.bin_sha1 = value
    else:
        if yaml_file != "":
            _logger.debug(f"file:{yaml_file} - key:{key} cannot be parsed")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: Copyright 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import re
from fosslight_util.oss_item import FileItem
from ._write_result_file import SHEET_NAME_SRC, SHEET_NAME_BIN, SHEET_NAME_BIN_YOCTO

const_other_proprietary_license = 'other proprietary license'
EXCLUDE_TRUE_VALUE = "Exclude"
IGNORE_COPYRIGHT = "NOASSERTION"
TLSH_CHECKSUM_NULL = "0"


class BinItem():
    def __init__(self, file_with_path, tlsh=TLSH_CHECKSUM_NULL, checksum=TLSH_CHECKSUM_NULL):
        self.source_name_or_path = file_with_path
        self.tlsh = tlsh
        self.checksum = checksum


class PackageItem(FileItem):
    def __init__(self):
        super().__init__("")
        self.oss_name = ""  # Default Value : Recipe Name
        self._name = ""  # oss name to print
        self._version = ""  # Default Value : PV
        self.license = []  # Default value : license, it will be overwritten with a pkg_license.
        self._declared_licenses = []  # Declared License in case of multi or dual licenses
        self._files = []  # Files in installed package - Value of "Binary Name"
        self.download_location = ""  # SRC_URI
        self.copyright = ""
        self.homepage = ""
        self.parent_package_name = ""  # Packages created at build time have different installed and parent package names.
        self._package_name = ""  # Installed Package name
        self.src_path = ""
        self.file_path = ""
        self.spdx_id = ""
        self._yocto_recipe = []
        self._yocto_package = []
        self.relative_path = ""
        self.additional_data = {}
        self.pv = ""
        self.pr = ""
        self._yocto_recipe = []
        self._yocto_package = []

    def __eq__(self, value):
        return self.spdx_id == value

    def __del__(self):
        pass

    @property
    def files(self):
        return self._files

    @files.setter
    def files(self, value):
        if not value:
            self._files = []
        else:
            if not isinstance(value, list):
                value = value.split(",")
            self._files.extend(value)
            self._files = [item.strip() for item in self._files]
            self._files = list(set(self._files))

    @property
    def yocto_recipe(self):
        return self._yocto_recipe

    @yocto_recipe.setter
    def yocto_recipe(self, value):
        if not isinstance(value, list):
            value = value.split(",")
        self._yocto_recipe.extend(value)
        self._yocto_recipe = [item.strip() for item in self._yocto_recipe]
        self._yocto_recipe = list(set(self._yocto_recipe))

    @property
    def yocto_package(self):
        return self._yocto_package

    @yocto_package.setter
    def yocto_package(self, value):
        if not isinstance(value, list):
            value = value.split(",")
        self._yocto_package.extend(value)
        self._yocto_package = [item.strip() for item in self._yocto_package]
        self._yocto_package = list(set(self._yocto_package))

    @property
    def package_name(self):
        return self._package_name

    @package_name.setter
    def package_name(self, value):
        self._package_name = value

    @property
    def copyright(self):
        return self._copyright

    @copyright.setter
    def copyright(self, value):
        if value:
            value = value.strip()
            if value == IGNORE_COPYRIGHT:
                value = ""
        self._copyright = value

    @property
    def oss_name(self):
        return self._oss_name

    @oss_name.setter
    def oss_name(self, value):
        if value != "":
            self._name = value.replace("lib32-", "", 1)
        self._oss_name = value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, value):
        str_value = str(value)
        if self._version != str_value:
            self._version = str_value
            _COMMIT_HASH = ["+gitAUTOINC+", "gitr+AUTOINC+", "+gitrAUTOINC+"]
            _TRUNCATE_WORD = ["-", "~", "+"]

            for word in _COMMIT_HASH:
                if word in str_value:
                    versions = str_value.split(word)
                    if len(versions) > 1:
                        self._version = versions[0]
                        if self.homepage == "":
                            self.homepage = self.download_location
                        if not self.download_location.endswith(";"):
                            self.download_location += ";"
                        self.download_location += f"commit:{versions[1]}"
                    break

            for word in _TRUNCATE_WORD:
                self._version = str_value.split(sep=word, maxsplit=1)[0]

    def set_license_flags(self, value):
        if value:
            self.comment = f"[NEED CHECK]LICENSE_FLAGS = {value}"

    @property
    def license(self):
        return self._license

    @license.setter
    def license(self, value, append_mode=False):

        if not append_mode:
            self._license = []
        if isinstance(value, list):
            for lic in value:
                self._license.append(update_license(lic))
        else:
            origin_lic = value

            matched_groups = re.findall(r'[^\)]+\)', value)
            license_list = ""
            for matched_group in matched_groups:
                matched_group = matched_group.strip()
                if re.search(r'^(?!like license)*(\s)*(\((.)*\))', matched_group):
                    matched_group = matched_group.replace('(', '')
                    matched_group = matched_group.replace(')', '')
                license_list += matched_group + "&"
            if license_list != "":
                value = license_list

            if '&' in value or '|' in value:
                self.comment = origin_lic
                value = value.replace('|', '&')
                license_list = value.split('&')
                for lic in license_list:
                    lic = lic.strip()
                    if lic != "":
                        self._license.append(update_license(lic))
            else:
                self._license.append(update_license(value))
        self._license = list(set(self._license))

        if len(self._license) > 1 and const_other_proprietary_license in self._license:
            self._license.remove(const_other_proprietary_license)

    @property
    def declared_licenses(self):
        return self._declared_licenses

    @declared_licenses.setter
    def declared_licenses(self, value):
        if value != "" and isinstance(value, list):
            if len(value) > 0:
                self.comment = "License changed to the license registered in OSC System DB."
                self._declared_licenses = value

    def get_print_item(self, sheet_name=SHEET_NAME_SRC, additional_column=[], binary_list=[]):
        print_items = []
        license_to_print = self.license
        exclude = EXCLUDE_TRUE_VALUE if self.exclude else ""
        if len(self.declared_licenses) > 0:
            license_to_print = self.declared_licenses
        if sheet_name == SHEET_NAME_BIN_YOCTO:
            row = [self.parent_package_name, self.oss_name, "", self.name, self.version,
                   ','.join(license_to_print), self.download_location, self.homepage,
                   self.copyright, exclude, self.comment]
            for column_name in additional_column:
                row.append(self.additional_data.get(column_name, ''))
            print_items.append(row)
        else:
            if sheet_name == SHEET_NAME_BIN:
                for pkg_file in self.files:
                    bin = next((n for n in binary_list if n.source_name_or_path == pkg_file),
                               BinItem(pkg_file, TLSH_CHECKSUM_NULL, TLSH_CHECKSUM_NULL))
                    row = [pkg_file, self.name, self.version, ','.join(license_to_print), self.download_location,
                           self.homepage, self.copyright, exclude, self.comment, bin.tlsh, bin.checksum]
                    for column_name in additional_column:
                        row.append(self.additional_data.get(column_name, ''))
                    print_items.append(row)

            elif sheet_name == SHEET_NAME_SRC:
                row = [self.parent_package_name, self.name, self.version, ','.join(license_to_print),
                       self.download_location,
                       self.homepage, self.copyright, exclude, self.comment]
                for column_name in additional_column:
                    row.append(self.additional_data.get(column_name, ''))
                print_items.append(row)
        return print_items


def update_license(license_name):
    license_map_list = {"bsd": "bsd-3-clause", "closed": const_other_proprietary_license, "mit-style": "mit-like license",
                        "bsd-style": "bsd-like license"}

    license_name = license_name.strip()
    license_name_ignore_case = license_name.lower()

    if license_name_ignore_case in license_map_list:
        return license_map_list[license_name_ignore_case]
    else:
        return license_name_ignore_case


def set_value_switch(oss, key, value, nested_pkg_name):
    if key == 'name' or key == 'oss_name':
        oss.oss_name = value
    elif key == 'version':
        oss.version = value
    elif key == 'source':
        oss.download_location = value
    elif key == 'version':
        oss.version = value
    elif key == 'copyright':
        oss.copyright = value
    elif key == 'license':
        oss.license = value
    elif key == 'file':
        oss.files = value
    elif key == 'exclude':
        oss.exclude = value
    elif key == 'comment':
        oss.comment = value
    elif key == 'homepage':
        oss.homepage = value
    elif key == 'src_path':
        oss.src_path = value
    elif key == 'file_path':
        oss.file_path = value
    elif key == 'license_flags':
        oss.set_license_flags(value)
    elif key == 'package':
        update_package_name(oss, value, nested_pkg_name)
    elif key == 'yocto_package':
        oss.yocto_package = value
    elif key == 'yocto_recipe':
        oss.yocto_recipe = value
    elif key == 'additional_data':
        oss.additional_data = value


def update_package_name(oss, value, nested_pkg_name):
    if oss.package_name != value:
        if value in nested_pkg_name:
            oss.parent_package_name = nested_pkg_name[value]
            oss.comment = f"Installed Package Name: {value}"
        else:
            oss.parent_package_name = value
        oss.package_name = value
    return oss



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: Copyright 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import fosslight_util.constant as constant
from fosslight_util.output_format import write_output_file

logger = logging.getLogger(constant.LOGGER_NAME)
OUTPUT_FILE_EXTENSION = ".xlsx"
HIDDEN_HEADER = {'TLSH', 'SHA1'}
SHEET_NAME_SRC = "SRC"
SHEET_NAME_BIN = "BIN"
SHEET_NAME_BIN_YOCTO = "BIN (Yocto)"


def write_result_from_bom(out_file_name, installed_packages_src, installed_packages_bin,
                          bin_android_mode=False, output_extension="", additional_column=[], binary_list=[], cover_item=""):
    SHEET_HEADER = {SHEET_NAME_BIN_YOCTO: ['ID', 'Binary Path', 'Source Path',
                                           'Notice', 'OSS Name', 'OSS Version',
                                           'License', 'Download Location', 'Homepage',
                                           'Copyright Text', 'Exclude', 'Comment'],
                    SHEET_NAME_SRC: ['ID', 'Source Path', 'OSS Name', 'OSS Version',
                                     'License', 'Download Location', 'Homepage', 'Copyright Text',
                                     'Exclude', 'Comment'],
                    SHEET_NAME_BIN: ['ID', 'Binary Path', 'OSS Name', 'OSS Version',
                                     'License', 'Download Location', 'Homepage',
                                     'Copyright Text', 'Exclude', 'Comment', 'TLSH', 'SHA1']}
    sheet_list = {}
    list_src_to_print = []
    list_bin_to_print = []
    logger.debug(f"write_result_from_bom - BIN(Yocto):{bin_android_mode}, {out_file_name}")
    if additional_column:
        for sheet_header_item in SHEET_HEADER.keys():
            SHEET_HEADER[sheet_header_item].extend(additional_column)

    src_sheet_name = SHEET_NAME_BIN_YOCTO if bin_android_mode else SHEET_NAME_SRC
    # remove_sheet_name = SHEET_NAME_SRC if bin_android_mode else SHEET_NAME_BIN_YOCTO
    # SHEET_HEADER.pop(remove_sheet_name)
    for scan_item in installed_packages_src:
        list_src_to_print.extend(scan_item.get_print_item(src_sheet_name, additional_column))
    list_src_to_print.insert(0, SHEET_HEADER[src_sheet_name])
    sheet_list[src_sheet_name] = list_src_to_print

    for scan_item in installed_packages_bin:
        list_bin_to_print.extend(scan_item.get_print_item(SHEET_NAME_BIN, additional_column, binary_list))
    if len(list_bin_to_print) > 0:
        list_bin_to_print.insert(0, SHEET_HEADER[SHEET_NAME_BIN])
        sheet_list[SHEET_NAME_BIN] = list_bin_to_print

    if sheet_list:
        cover_item.external_sheets = sheet_list
    success_to_write, msg, result_file = write_output_file(out_file_name, output_extension,
                                                           cover_item, {}, HIDDEN_HEADER)

    if success_to_write:
        logger.info(f"Output file :{result_file}")
    else:
        logger.error(f"Fail to generate result file.: {msg}")


def print_src_analysis_result(recipe_list, output_file, cover_item):
    SHEET_NAME = "Source_Analysis"
    SRC_HEADER = {SHEET_NAME: ["No", "OSS Name", "OSS Version", "Check Point", "License_OSS_Report",
                               "Detected license from ScanCode", "Download location"]}

    try:
        if len(recipe_list.keys()) > 0:
            sheet_list = {}
            list_to_print = []

            for key, item in recipe_list.items():
                list_to_print.append([item['name'], item['version'], item['comment'], ','.join(item['license']),
                                      ','.join(item['license_detected']), item['link']])

            sheet_list[SHEET_NAME] = list_to_print
            cover_item.external_sheets = sheet_list
            success_to_write, msg, result_file = write_output_file(output_file, OUTPUT_FILE_EXTENSION,
                                                                   cover_item, SRC_HEADER)
            # success_to_write, writing_msg, result_file = write_output_file(output_file, OUTPUT_FILE_EXTENSION, sheet_list, SRC_HEADER)

            if success_to_write:
                logger.info(f"Source Analysis - Writing Output file ({result_file}), Success: {success_to_write}")
            else:
                logger.error(f"Fail to generate result file. msg:({msg})")
    except Exception as ex:
        logger.error(f'write_result_from_source_analysis:{ex}')
